#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// Lee el valor de los parametros y los asigna a las variables:
// cfile1   - Nombre de la matriz
// cfile1   - Nombre del vector
void readParams(int argc, char **argv, char *cfile1, char *cfile2) {
    if(argc>1) strcpy(cfile1, argv[1]);
    if(argc>2) strcpy(cfile2, argv[2]);
}

// Lectura del vector en el archivo cfile.
// Devuelve NULL si no se pudo abrir el archivo.
double *readVector(char *cfile, int *nr) {
    double   *vec;
    FILE       *f1 = fopen(cfile, "rb");
    
    if(!f1) return(NULL);
    fread(nr, sizeof(int), 1, f1);
    vec = (double *) malloc( (*nr)*sizeof(double));
    if(vec==NULL) return(NULL);
    fread(vec, sizeof(double), *nr, f1);
    fclose(f1);
    return(vec);
}

// Escritura del vector vec en el archivo cfile.
// Devuelve 0 en caso de exito y 1 si no.
int writeVector(double *vec, int dim, char *cfile) {
    FILE       *f1 = fopen(cfile, "wb");
    
    if(!f1) return(1);
    fwrite(&dim, sizeof(int), 1, f1);
    fwrite(vec, sizeof(double), dim, f1);
    fclose(f1);
    return(0);
}


// Imprime en la consola las entradas del vector
void printVector(double *vec, int dim) {
    int i; 
    for(i=0; i<dim; ++i) 
	    printf("% 8.4f   ", vec[i]);
	printf("\n");
}

// Reserva memoria para una matriz de tama~no nr x nc
double **createMatrix(int nr, int nc) {
    int      i;
    double **mat;

    // Reservamos memoria
    mat    = (double **) malloc( (nr)*sizeof(double *));
    if(mat==NULL) return(NULL);
    mat[0] = (double *) malloc( nr*nc*sizeof(double));
    if(mat[0]==NULL) return(NULL);
    for(i=1; i<nr; ++i) 
        mat[i] = mat[i-1] + nc;
    return(mat);
}


// Libera la memoria del arreglo bidimensional
void freeMatrix(double **mat) {
    free(mat[0]);
    free(mat);
}

// Lectura de la matriz en el archivo cfile.
// Devuelve NULL si no se pudo abrir el archivo.
double **readMatrix(char *cfile, int *nr, int *nc) {
    double  **mat;
    FILE     *f1 = fopen(cfile, "rb");
    
    if(!f1) return(NULL);
    // Lectura del tama~no de la matriz
    fread(nr, sizeof(int), 1, f1);
    fread(nc, sizeof(int), 1, f1);
    // Reservamos memoria
    mat    = createMatrix(*nr, *nc);
    // Lectura de los datos
    fread(mat[0], sizeof(double), (*nr)*(*nc), f1);
    fclose(f1);
    return(mat);
}

// Escritura de la matriz mat en el archivo cfile.
// Devuelve 0 en caso de exito y 1 si no.
int writeMatrix(double **mat, int nr, int nc, char *cfile) {
    FILE       *f1 = fopen(cfile, "wb");
    
    if(!f1) return(1);
    fwrite(&nr, sizeof(int), 1, f1);
    fwrite(&nc, sizeof(int), 1, f1);
    fwrite(mat[0], sizeof(double), nr*nc, f1);
    fclose(f1);
    return(0);
}


// Imprime en la consola las entradas de la matriz
void printMatrix(double **mat, int nr, int nc) {
    int i, j; 
    for(i=0; i<nr; ++i) {
        for(j=0; j<nc; ++j) 
            printf("% 6.2f   ", mat[i][j]);
        printf("\n");
    }
}

int main(int argc, char **argv)      {
    char    cfile1[50], cfile2[50];

    // Lectura de los parametros
    readParams(argc, argv, cfile1, cfile2);

    // Lectura e impresion de la matriz
    int     nr, nc, dim;
    double **mat= readMatrix(cfile1, &nr, &nc);
    double *vec = readVector(cfile2, &dim);
    if(vec==NULL) {
        printf("El archivo %s no puedo ser leido.\n", cfile2);
        return(0);
    }
    if(mat==NULL) {
        printf("El archivo %s no puedo ser leido.\n", cfile1);
        return(0);
    }

    printf("\nVector :\n" );
    printVector(vec, dim);
    printf("\nMatrix:\n" );
    printMatrix(mat, nr , nc);


    free(vec);
    freeMatrix(mat);
    return(0);
}
// Para compilar
// gcc -o lecturaBinarios lecturaBinarios.c -lm 

// Para ejecutar
// ./lecturaBinarios mat3x4.bin vec_mat3x4.bin






