#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// Lee el valor de los parametros y los asigna a las variables:
// cdat   - Nombre del archivo de datos
// dft    - factor de escala
void readParams(int argc, char **argv, char *cdat1, char *cdat2) {
    if(argc>1) strcpy(cdat1, argv[1]);
    if(argc>2) strcpy(cdat2, argv[2]);
}


// Reserva memoria para una matriz de tama~no nr x nc
double **createMatrix(int nr, int nc) {
    int      i;
    double **mat;

    // Reservamos memoria
    mat    = (double **) malloc( (nr)*sizeof(double *));
    if(mat==NULL) return(NULL);
    mat[0] = (double *) malloc( nr*nc*sizeof(double));
    if(mat[0]==NULL) return(NULL);
    for(i=1; i<nr; ++i) 
        mat[i] = mat[i-1] + nc;
    // Lectura de los datos
    return(mat);
}

// Lee un archivo de texto que contiene las
// entradas de un arreglo bidimensional, que tiene el
// formato especificado.
// Devuelve el apuntador al espacio de memoria que tiene
// la informacion y el numero nr de filas y nc de columnas
double **readMatrixTxt(const char* fileName, int *nr, int *nc) {
    FILE *fp = fopen(fileName, "rt");   
    double val;
    int    i, j;
    if (!fp) return NULL;

    fscanf(fp, "%d %d", nr, nc);
    double **mat = createMatrix(*nr, *nc);

    for(i=0; i<*nr; i++) {
        for(j=0; j<*nc; j++) {
            fscanf(fp, "%lf", &val);
            mat[i][j] = val;
        }
    }
    return mat;
}


// Escritura de la matriz mat en el archivo cfile.
// Devuelve 0 en caso de exito y 1 si no.
int writeMatrixBin(double **mat, int nr, int nc, char *cfile) {
    FILE       *f1 = fopen(cfile, "wb");
    
    if(!f1) return(1);
    fwrite(&nr, sizeof(int), 1, f1);
    fwrite(&nc, sizeof(int), 1, f1);
    fwrite(mat[0], sizeof(double), nr*nc, f1);
    fclose(f1);
    return(0);
}

// Libera la memoria del arreglo bidimensional
void freeMatrix(double **mat) {
    free(mat[0]);
    free(mat);
}


// Imprime en la consola las entradas de la matriz
void printMatrix(double **mat, int nr, int nc) {
    int i, j; 
    for(i=0; i<nr; ++i) {
        for(j=0; j<nc; ++j) 
            printf("% 6.2f   ", mat[i][j]);
        printf("\n");
    }
}

int main(int argc, char **argv)      {
    char    cfile1[80], cfile2[80], cfile3[80];

    // Lectura de los parametros
    readParams(argc, argv, cfile1, cfile2);

    int      nr, nc;
    double **matr = readMatrixTxt(cfile1, &nr, &nc);

    printf("Lectura del archivo %s: \n", cfile1);
    printMatrix(matr, nr, nc);

    printf("\nGenerando el archivo binario %s. \n", cfile2);
    writeMatrixBin(matr, nr, nc, cfile2);

    freeMatrix(matr);

    return(0);
}
// Para compilar
// gcc -o matTxt2Bin  matTxt2Bin.c  -lm

// Para ejecutar
// ./matTxt2Bin mat3x4.txt  mat3x4.bin 


